﻿using UnityEngine;
using System.Collections;

public class snowman : MonoBehaviour
{
    public float _speed = 2f;

    public bool _is_player = false;

    public Transform _enemy_goal;
    public Transform _own_goal;

    public Transform _body;
    public Transform _feet;

    public float _kick_force = 2000f;

    private Vector3 _body_movement = new Vector3(0f, 0f, 0f);
    private float _body_movement_factor = 0.003f;
    private float _body_movement_rand = 0f;
    private float _feet_speed = 3f;

    private float _dir = 1;
    private Vector3 _dir_vec = new Vector3(1f, 1f, 1f);

    private GameObject _ball;

    private bool _can_kick;

    private Vector2 _move = new Vector2(0f, 0f);

    private Vector3 _start_pos;

    public Sprite[] _walk_sprites;
    private int _walk_sprites_num;
    private int _walk_sprite_index = 0;
    private float _walk_sprites_duration_max = 0.2f;
    private float _walk_sprites_duration;
    private Sprite _idle_sprite;
    private SpriteRenderer _sprite_renderer;
   
    void Start()
    {
        _start_pos = transform.localPosition;

        _can_kick = true;
        _body_movement_rand = Random.Range(5f, 15f);

        _sprite_renderer = _body.GetComponent<SpriteRenderer>();

        _idle_sprite = _sprite_renderer.sprite;

        _walk_sprites_duration = _walk_sprites_duration_max;
        _walk_sprites_num = _walk_sprites.Length;
    }

    public void reset()
    {
        transform.localPosition = new Vector3(_start_pos.x, _start_pos.y, _start_pos.z);
    }

    void Update()
    {
        if (rigidbody2D.velocity.x != 0f)
            _dir = (rigidbody2D.velocity.x >= 0 ? 1f : -1f);

        _body_movement.Set(0f, Mathf.Sin(Time.timeSinceLevelLoad * _body_movement_rand) * _body_movement_factor, 0f);
        _body.transform.Translate(_body_movement);
        if (rigidbody2D.velocity.x > 0.2f || rigidbody2D.velocity.x < -0.2f || rigidbody2D.velocity.y > 0.2f || rigidbody2D.velocity.y < -0.2f)
        {
            _feet.transform.Rotate(Vector3.forward, (_feet_speed * -_dir) % 360);

            _walk_sprites_duration -= Time.deltaTime;
            if (_walk_sprites_duration <= 0)
            {
                _walk_sprite_index = (_walk_sprite_index + 1) % _walk_sprites_num;
                _walk_sprites_duration = _walk_sprites_duration_max;
            }
            _sprite_renderer.sprite = _walk_sprites[_walk_sprite_index];
        }
        else
        {
            _sprite_renderer.sprite = _idle_sprite;
        }

        if (_dir != 0)
            _dir_vec.Set(-_dir, 1f, 1f);
        _body.transform.localScale = _dir_vec;
	}

    void kick(float force_factor = 1f)
    {
        if (_can_kick && _ball != null)
        {
            _ball.rigidbody2D.AddForce(rigidbody2D.velocity * _kick_force);
            _ball.SendMessage("kicked");
            _can_kick = false;
            StartCoroutine(can_kick_wait());
        }
    }

    public void set_ball(GameObject ball)
    {
        _ball = ball;
    }

    public void lost_ball()
    {
        StartCoroutine(lost_ball_());
    }

    private IEnumerator lost_ball_()
    {
        yield return new WaitForSeconds(0.3f);
        _ball = null;
    }

    IEnumerator can_kick_wait()
    {
        yield return new WaitForSeconds(1f);
        _can_kick = true;
    }

    public bool has_ball()
    {
        return (_ball != null);
    }

    public Transform get_goal()
    {
        return _enemy_goal;
    }

    public void move_to_ball(Transform ball)
    {
        _move.Set(ball.transform.localPosition.x - transform.localPosition.x, ball.transform.localPosition.y - transform.localPosition.y);
        _move.Normalize();
        _move *= _speed;
        rigidbody2D.velocity = _move;
    }

    public void move_to_goal()
    {
        _move.Set(_enemy_goal.transform.localPosition.x - transform.localPosition.x, _enemy_goal.transform.localPosition.y - transform.localPosition.y);
        _move.Normalize();
        _move *= _speed;
        rigidbody2D.velocity = _move;
    }
    
    public void move_to_own_goal()
    {
        _move.Set(_own_goal.transform.localPosition.x - transform.localPosition.x, _own_goal.transform.localPosition.y - transform.localPosition.y);
        _move.Normalize();
        _move *= _speed * 0.7f;
        rigidbody2D.velocity = _move;
    }

    public void shoot_at_goal(bool slow = false)
    {
        move_to_goal();

        if (_ball != null)
        {
            kick((slow ? Random.Range(0.25f, 0.4f) : 1f));
        }
    }

    public void pass_to_team(Transform team_member)
    {
        _move.Set(team_member.transform.localPosition.x - transform.localPosition.x, team_member.transform.localPosition.y - transform.localPosition.y + Random.Range(-10f, 10f));
        _move.Normalize();
        _move *= _speed;
        rigidbody2D.velocity = _move;
        kick();
    }

    public void move_better_pos(ai_manager.move_data md)
    {
        if (md.team_owns_ball)
        {
            float goal_dist = Vector3.Distance(_enemy_goal.transform.localPosition, transform.localPosition);
            bool b1 = Vector3.Dot(md.ball.transform.localPosition, _own_goal.transform.localPosition) > 0;
            if (goal_dist > 6f && b1)
                move_to_goal();
            else
            {
                float player_dist = Vector3.Distance(md.nearest_player.transform.localPosition, transform.localPosition);
                Vector2 player_dir = md.nearest_player.transform.rigidbody2D.velocity;
                if (player_dist > 2)
                    player_dir *= Random.Range(10f, 15f);
                else
                    player_dir *= Random.Range(2f, 5f);
                _move.Set(player_dir.x - transform.localPosition.x, player_dir.y - transform.localPosition.y);
                _move.Normalize();
                _move *= _speed;
                rigidbody2D.velocity = _move;
            }
        }
        else
        {
            float ball_dist = Vector3.Distance(md.ball.transform.localPosition, transform.localPosition);
            float goal_dist = Vector3.Distance(_own_goal.transform.localPosition, transform.localPosition);
            bool b2 = Vector3.Dot(md.ball.transform.localPosition, _own_goal.transform.localPosition) > 0;
            if (goal_dist > 7f && b2 && ball_dist > 5f)
                move_to_own_goal();
            else if (ball_dist < 5f && b2)
                move_to_ball(md.ball);
        }
    }

    public void set_dir(float val)
    {
        _dir = val;
    }
}
